#include <SD.h>
#include <SPI.h>
#include "TMRpcm.h"

#define slaveSelect 10    //Slave Select/Chip select pi of the SD card module
#define playRand 2        //Play Random music button Input
#define record 3          //Record audio button input
#define mic A0            //Mic analog input pin

File root;
TMRpcm player;

int audio_files = 0;
int flag = 0;

void setup() {
  Serial.begin(9600);

  Serial.println("...Reading Audio Files...");

  if (!SD.begin(slaveSelect)){
      Serial.println("failed!");
      while(1);
    }

  Serial.println("DONE!");

  player.speakerPin = 9;   // PWM pin for audio output
  player.setVolume(5);    // 0 to 7 volume level for output
  player.quality(1);     

  pinMode(playRand, INPUT);
  pinMode(record, INPUT);
  pinMode(mic, INPUT);

  root = SD.open("/");      //open SD card root directory as a 'root' file object
  printAudioFiles();
  if(SD.exists("record.wav")){
      SD.remove("record.wav");
      Serial.println("Previous Recording Removed!");
    }
}

void loop() {
  root.rewindDirectory();
  int playButtonState = digitalRead(playRand);  // Read play button state
  int recordButtonState = digitalRead(record);  // Read record button state

  if(player.isPlaying()){
      // if any audio file is still playing print PLAYING AUDIO
      Serial.println("PLAYING AUDIO");
    }

  //invoke playRandAudio() function when play button is pressed
  if(playButtonState == HIGH){
      Serial.println("");
      Serial.println("Playing Random Audio");
      playRandAudio();
      delay(500);
    } 

  while(recordButtonState == HIGH){
      if(flag == 0){
        SD.remove("record.wav");
        Serial.println("");
        Serial.println("...Start Speaking...");
        player.startRecording("record.wav", 16000, mic);
        flag = 1;
      }
      recordButtonState = digitalRead(record);
    }
  if(flag == 1){
      player.stopRecording("record.wav");
      Serial.println("...Recording Stopped!!...");
      Serial.println("...Playing the Recorded Audio...");
      player.play("record.wav");
      flag = 0;
      
    }
}


// Funtion to print the name of all .wav files present in the SD card
void printAudioFiles(){
    while(1){
        File entry = root.openNextFile();
        if(!entry){
            // exit loop if no more files are left to read
            break;
          }
        
        uint8_t nameLen = String(entry.name()).length();    //get the length of the file name
        String ext = String(entry.name()).substring(nameLen - 4);    // save the extension of the file in 'ext' string
        
        // check for .wav files in sd card and print their name
        if(ext.equalsIgnoreCase(".wav")){
            Serial.println(entry.name());
            audio_files = audio_files + 1;
          }
        else{
            // close this file if its not .wav file
            entry.close();
          }
          
      }
      Serial.println("");
  }

  
// Funtion to randomly pick an audio file from the sd card and play it
void playRandAudio(){
    // generate a random file name
    int randomNum = random(audio_files);
    String file_name = String("audio_");
    file_name += randomNum+1;
    file_name += ".wav";

    Serial.println("");
    Serial.println("Random File Chosen: ");
    Serial.println(file_name);
    
    while(1){
        File entry = root.openNextFile();
        if(!entry){
            // exit loop if no more files are left to read
            break;
          }
        
        // check if name of current file is same as the randomly generated file
        if(file_name.equalsIgnoreCase(String(entry.name()))){
            Serial.println("...Playing File...");
            Serial.println(entry.name());
            player.play(entry.name());
            return;
          }
        else{
            // close this file if its not .wav file
            entry.close();
          }
          
    }
      Serial.println("");
      return;
}
